#include "quaternion.h"
#include "vector.h"
#include "matrix3x3.h"
#include <wx/math.h>
#include <float.h>

Quaternion::Quaternion(double x, double y, double z, double w)
	:
	x(x),
	y(y),
	z(z),
	w(w)
{
}

Quaternion::Quaternion(const Vector &axis, double angle)
{
	set(axis, angle);
}

void Quaternion::set(const Vector &axis, double angle)
{
	double norm = axis.norm();

	if( norm < 1E-10 )
	{
		// Axis is too short, no rotation
		x = y = z = 0.;
		w = 1;
	}
	else
	{
		angle /= 2.;
		const double factor = sin( angle ) / norm;
		x = axis.x * factor;
		y = axis.y * factor;
		z = axis.z * factor;
		w = cos( angle );
	}
}

Quaternion & Quaternion::operator+=(const Quaternion &q)
{
	for( int i = 0; i < 4; i++ )
		val[i] += q.val[i];
	return *this;
}

Quaternion Quaternion::operator-() const
{
	return Quaternion(-x, -y, -z, w);
}

Quaternion Quaternion::operator+(const Quaternion &q) const
{
	return Quaternion(
		x + q.x,
		y + q.y,
		z + q.z,
		w + q.w
		);
}

Quaternion Quaternion::operator-(const Quaternion &q) const
{
	return Quaternion(
		x - q.x,
		y - q.y,
		z - q.z,
		w - q.w
		);
}

Quaternion Quaternion::operator*(const Quaternion &q) const
{
	return Quaternion(
		w * q.x + x * q.w + y * q.z - z * q.y,
		w * q.y + y * q.w + z * q.x - x * q.z,
		w * q.z + z * q.w + x * q.y - y * q.x,
		w * q.w - x * q.x - y * q.y - z * q.z);
}

Quaternion & Quaternion::operator*=(const Quaternion &q)
{
	*this = operator*(q);
	return *this;
}

Quaternion Quaternion::operator*(double s) const
{
	return Quaternion(
		s * x,
		s * y,
		s * z,
		s * w
		);
}

Quaternion Quaternion::operator/(double s) const
{
	return *this * (1. / s);
}

//! Rotates a vector
Vector Quaternion::operator*(const Vector &v) const
{
	Matrix3x3 m;
	m.setRotation(*this);
	return m * v;
}

double Quaternion::squaredNorm() const
{
	return x*x + y*y + z*z + w*w;
}

double Quaternion::norm() const
{
	return sqrt( squaredNorm() );
}

void Quaternion::normalize()
{
	double l = norm();

	for( int i = 0; i < 4; i++ )
		val[i] /= l;
}

Quaternion Quaternion::normalized() const
{
	return *this / norm();
}
