#include "vector.h"
#include "angle.h"
#include <wx/math.h>

Vector::Vector(double x, double y, double z)
	: x(x), y(y), z(z)
{
}

//! Build a vector from 2 points (double vector)
Vector::Vector(const Vertex &a, const Vertex &b)
{
	for(int i = 0; i < 3; i++)
		val[i] = b[i] - a[i];
}

//! Build a vector from 2 points (float array)
Vector::Vector(const float *a, const float *b)
{
	for(int i = 0; i < 3; i++)
		val[i] = b[i] - a[i];
}

Vector Vector::operator-() const
{
	return Vector(
		-x,
		-y,
		-z
		);
}

Vector Vector::operator+ (const Vector &v) const
{
	return Vector(
		x + v.x,
		y + v.y,
		z + v.z
		);
}

Vector Vector::operator-(const Vector &v) const
{
	return Vector(
		x - v.x,
		y - v.y,
		z - v.z
		);
}

//! Adds a vector to the first one: u = u + v
Vector& Vector::operator+= (const Vector &v)
{
	x += v.x;
	y += v.y;
	z += v.z;
	return *this;
}

//! Substracts a vector to the first one: u = u - v
Vector& Vector::operator-= (const Vector &v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	return *this;
}

Vector Vector::operator*(double mult) const
{
	return Vector(
		x * mult,
		y * mult,
		z * mult
		);
}

Vector Vector::operator/(double div) const
{
	return Vector(
		x / div,
		y / div,
		z / div
		);
}

Vector& Vector::operator*=(double mult)
{
	x *= mult;
	y *= mult;
	z *= mult;
	return *this;
}

Vector& Vector::operator/=(double div)
{
	return operator*=( 1. / div );
}

//! Scalar product (dot product)
double Vector::operator*(const Vector &v) const
{
	return 	x * v.x +
		y * v.y +
		z * v.z;
}

//! Cross product
Vector Vector::operator/(const Vector &v) const
{
	return Vector(
		y * v.z - z * v.y,
		z * v.x - x * v.z,
		x * v.y - y * v.x
		);
}

double Vector::operator[](unsigned int index) const
{
	return coord[index];
}

double& Vector::operator[](unsigned int index)
{
	return coord[index];
}

//! Squared norm
double Vector::sqNorm() const
{
	return x*x + y*y + z*z;
}

//! Norm
double Vector::norm() const
{
	return sqrt( sqNorm() );
}

Vector& Vector::normalize()
{
	return operator/= ( norm() );
}

//! Returns a normalized vector
Vector Vector::normalized() const
{
	return operator/ ( norm() );
}

/*! Returns the corresponding vector in spherical coordinates
 *  (radius, latitude, longitude), or
 *  (radius, pitch, yaw) using Euler angles terminology.
 */
Vector Vector::cartesianToSpherical() const
{
	double radius = norm();
	return Vector(
		radius,
		asin( z / radius ),	// latitude = pitch
		atan2( y, x )		// longitude = yaw
		);
}

/*! Returns the corresponding vector in cartesian coordinates,
 *  assuming that the current vector is defined as follows:
 *  (radius, latitude, longitude), or
 *  (radius, pitch, yaw) using Euler angles terminology.
 */
Vector Vector::sphericalToCartesian() const
{
	// Distance from the origin to the projection on the X-Y plane
	double xyDist = radius * cos(latitude);

	return Vector(
		xyDist * cos(longitude),
		xyDist * sin(longitude),
		radius * sin(latitude)
		);
}

/*! Returns the vector in cartesian coordinates corresponding to the given
 *  spherical parameters.
 */
Vector Vector::sphericalToCartesian( double radius, const Angle &latitude, const Angle &longitude )
{
	return Vector(
			radius,
			latitude.radians,
			longitude.radians
		).sphericalToCartesian();
}


Vector2::Vector2(double x, double y)
	: x(x), y(y)
{}
